defmodule Pandoc.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  require Logger
  import Pandoc, only: [latest_version: 0, configured_version: 0, bin_version: 0]

  @doc false
  def start(_, _) do
    if Application.get_env(:pandoc, :version_check, true) do
      unless Application.get_env(:pandoc, :version) do
        Logger.warning("""
        pandoc version is not configured. Please set it in your config files:

            config :pandoc, :version, "#{latest_version()}"
        """)
      end

      configured_version = configured_version()

      case bin_version() do
        {:ok, ^configured_version} ->
          :ok

        {:ok, version} ->
          Logger.warning("""
          Outdated pandoc version. Expected #{configured_version}, got #{version}. \
          Please run `mix pandoc.install` or update the version in your config files.\
          """)

        :error ->
          :ok
      end
    end

    Supervisor.start_link([], strategy: :one_for_one, name: Pandoc.Supervisor)
  end
end
